"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
const subscription_1 = __importDefault(require("./subscription"));
const utils = require('./utils');
const Connection = require('./connection');
class SEPA {
    get config() {
        return this._config;
    }
    constructor(parameters) {
        this._config = parameters;
        this.queryURI = utils.createURI(parameters.sparql11protocol.protocol, parameters.host, parameters.sparql11protocol.port, parameters.sparql11protocol.query.path);
        this.updateURI = utils.createURI(parameters.sparql11protocol.protocol, parameters.host, parameters.sparql11protocol.port, parameters.sparql11protocol.update.path);
        this.connectionPool = new Map();
        let subprotcol = parameters.sparql11seprotocol.protocol;
        let selectSubProtocol = parameters.sparql11seprotocol.availableProtocols[subprotcol];
        this.subscribeURI = utils.createURI(subprotcol, parameters.host, selectSubProtocol.port, selectSubProtocol.path);
        this._wsFactory = (uri) => {
            return new isomorphic_ws_1.default(uri);
        };
    }
    query(query, config) {
        let q_uri = this.queryURI;
        // clone the configuration
        let axiosConfig = mergeConfigurations({}, this._config);
        if (config !== undefined) {
            axiosConfig = mergeConfigurations(this._config, config);
            q_uri = utils.createURI(axiosConfig.sparql11protocol.protocol, axiosConfig.host, axiosConfig.sparql11protocol.port, axiosConfig.sparql11protocol.query.path);
        }
        axiosConfig = setHeadersIfUndefined(axiosConfig, { "Content-Type": "application/sparql-query" });
        return axios_1.default.post(q_uri, query, axiosConfig.options).then(function (response) {
            return response.data;
        });
    }
    update(update, config) {
        let up_uri = this.updateURI;
        // clone the configuration
        let axiosConfig = mergeConfigurations({}, this._config);
        if (config !== undefined) {
            axiosConfig = mergeConfigurations(this._config, config);
            up_uri = utils.createURI(axiosConfig.sparql11protocol.protocol, axiosConfig.host, axiosConfig.sparql11protocol.port, axiosConfig.sparql11protocol.update.path);
        }
        axiosConfig = setHeadersIfUndefined(axiosConfig, { "Content-Type": "application/sparql-update" });
        return axios_1.default.post(up_uri, update, axiosConfig.options).then(function (response) {
            return { "status": response.status,
                "statusText": response.statusText
            };
        });
    }
    subscribe(query, config, alias) {
        let sub_uri = this.subscribeURI;
        let subConfig = this._config;
        if (config !== undefined) {
            subConfig = mergeConfigurations(this._config, config);
            // delete https agent if present
            subConfig.options && subConfig.options.httpsAgent && delete subConfig.options.httpsAgent;
            let subprotcol = subConfig.sparql11seprotocol.protocol;
            let selectSubProtocol = subConfig.sparql11seprotocol.availableProtocols[subprotcol];
            sub_uri = utils.createURI(subprotcol, subConfig.host, selectSubProtocol.port, selectSubProtocol.path);
        }
        let connection = this.connectionPool.get(sub_uri);
        if (!connection) {
            connection = new Connection(sub_uri, this._wsFactory);
            this.connectionPool.set(sub_uri, connection);
            connection.on("close", (() => { this.connectionPool.delete(sub_uri); }).bind(this));
            connection.on("error", () => {
                /* Ignore the error, connection will try to reconnect automatically.
                   TODO: this is a bad pattern, we should design connection error handling better
                   See https://github.com/arces-wot/SEPA-js/issues/36
                */
            });
        }
        let options = subConfig ? subConfig.options : {};
        return new subscription_1.default(query, connection, alias, options);
    }
}
exports.default = SEPA;
function setHeadersIfUndefined(localConfig, headers) {
    let config = Object.assign({}, localConfig);
    if (!config.options)
        config.options = { headers: {} };
    if (!config.options.headers)
        config.options.headers = {};
    Object.keys(headers).forEach((key) => {
        if (!config.options.headers.hasOwnProperty(key)) {
            config.options.headers[key] = headers[key];
        }
    });
    return config;
}
function mergeConfigurations(defaults, user) {
    return utils.mergeWithDefaults(defaults, user);
}
//# sourceMappingURL=sepa.js.map